# Top level namespace
module GreenHat
  # Helper for all things user environment / settings / history
  module Settings
    def self.settings
      @settings ||= {
        history: [],
        assume_json: true,
        fuzzy_file_match: true,

        # round: [2],
        # page: [:true] Automatic,
        truncate: TTY::Screen.width * 4,
        color: true
      }
    end

    def self.assume_json?
      settings.assume_json
    end

    # Allow for future disabling of color output
    def self.color?
      settings.color
    end

    # Load User Settings and drop them into settings
    def self.settings_load
      return true unless File.exist?(settings_file)

      Oj.load(File.read(settings_file)).each do |key, value|
        settings[key] = value
      end
    end

    def self.settings_file
      "#{dir}/settings.json"
    end

    # Set any Log Arguments that weren't set otherwise | Conditional assign applied on bool
    # rubocop:disable Style/GuardClause
    def self.default_log_flags(flags, skip_flags)
      flags[:round] = settings.round if !(skip_flags.include?(:round) || flags.key?(:round)) && settings.round

      flags[:page] = settings.page if !(skip_flags.include?(:page) || flags.key?(:page)) && settings.page

      flags[:truncate] = settings.truncate unless skip_flags.include?(:truncate) || flags.key?(:truncate)

      # Fuzzy File Match
      unless skip_flags.include?(:fuzzy_file_match) || flags.key?(:fuzzy_file_match)
        flags[:fuzzy_file_match] = settings.fuzzy_file_match
      end
    end
    # rubocop:enable Style/GuardClause

    def self.start
      Dir.mkdir dir unless Dir.exist? dir

      # Load User Settings
      settings_load

      # CMD History Loading / Tracking
      File.write(cmd_file, "\n") unless File.exist? cmd_file
    end

    def self.dir
      "#{ENV['HOME']}/.greenhat"
    end

    # ----------------------------------
    # Command History
    def self.cmd_history
      "#{dir}/cmd_history"
    end

    def self.cmd_file
      "#{dir}/cmd_history"
    end

    def self.cmd_history_clean
      File.read(cmd_file).split("\n")
    end

    def self.cmd_history_clear
      File.write(cmd_file, "\n")
    end

    def self.cmd_add(line)
      File.write(cmd_file, line, File.size(cmd_file), mode: 'a')
    end

    # ----------------------------------

    # File Load History
    def self.history_file
      "#{dir}/file_history"
    end
  end
end
