# Top Level what everything is
class Thing < Teron
  include GreenHat::Formatters
  include GreenHat::ThingHelpers
  include GreenHat::Spinner
  include GreenHat::Kind
  include GreenHat::FileTypes
  include ActionView::Helpers::DateHelper
  include ActionView::Helpers::NumberHelper

  belongs_to :host
  belongs_to :archive

  # File
  field :file # Full File Path / File Load
  field :name # Base File Name
  field :path # Symbol Name Helper
  field :size # Total File Size

  # Classification
  field :kind # File Information
  field :type # Shared Naming Convention (Based off of SOS)
  field :log # Parsable / Include in log formats

  # Preloader Flags
  field :parsed # Flag for Parsing
  field :result # Processed Data
  field :raw_result # Flag for reading raw data

  def friendly_name
    "#{archive.friendly_name.pastel(:blue)} #{name.pastel(:green)}"
  end

  def setup
    self.name = build_path('/')
    self.path = build_path.gsub(/[^\w\s]/, '_')
    self.size = number_to_human_size File.size(file)

    kind_collect # Set Kind and Type
    kind_setup

    save!
  end

  # Processor
  def data
    process unless parsed

    result
  end

  def raw
    raw_read if raw_result.nil?

    raw_result
  end

  # Filter / Archive Pattern Matching
  def archive?(archive_names)
    archive_names.map(&:to_s).any? { |x| archive.name.include? x }
  end

  # Helper for empty logs
  def blank?
    data.blank?
  end

  def raw_read
    spin_start("Read  #{name.pastel(:blue)} #{size.pastel(:bright_black)}")
    self.raw_result = File.read(file).split("\n")
  rescue StandardError => e
    LogBot.fatal('Raw Read', message: e.message, backtrace: e.backtrace.first)
    self.raw_result = ''
  ensure
    spin_done
  end

  def output(print_it = true)
    if print_it
      puts raw.join("\n")
    else
      raw
    end
  end

  # Log Identifier Helper
  def log?
    GreenHat::SuperLog.log? kind
  end

  # Hashed values searching
  def processed?
    kind != :raw
  end

  # Helper Formatter Method
  def formatter
    "format_#{kind}".to_sym
  end

  def process
    # Read First if Needed (Spinner)
    raw_read if raw_result.nil?

    if methods.include? formatter
      spin_start("Parse #{name.pastel(:blue)} #{kind.to_s.pastel(:bright_black)} ")
      begin
        send(formatter)
      rescue StandardError => e
        LogBot.fatal('Process', message: e.message, backtrace: e.backtrace.first)
      end
      spin_done
    else
      LogBot.fatal('Thing', "No Formatter for #{formatter}")
    end

    self.parsed = true

    save!
  end
end
