# Top
module GreenHat
  # Log
  module Formatters
    # ==========================================================================
    # Formatters
    # Fallback to String
    # ==========================================================================
    def format_json
      self.result = raw.map do |row|
        result = begin
          Oj.load row
        rescue EncodingError
          { message: row }
        end

        # Parsing Time
        format_json_traverse result

        result.sort.to_h
      rescue StandardError => e
        # TODO: Background Logger?
        e.message
        LogBot.warn('JSON Parse', e.message)
        next
      end

      :ok
    end
    # ==========================================================================

    # Recursively Navigate
    def format_json_traverse(result)
      format_json_time(result)

      result.each do |_key, value|
        next unless value.instance_of? Hash

        format_json_traverse(value)
      end
    end

    # Check for Common Fields
    def format_json_time(result)
      result.time = format_time_parse(result.time) if result.key? :time
      result.created_at = format_time_parse(result.created_at) if result.key? :created_at
      result.enqueued_at = format_time_parse(result.enqueued_at) if result.key? :enqueued_at
    rescue StandardError => e
      LogBot.warn('JSON Time Parse', e.message)
      true
    end

    # Handle Epoch Timestamps as well as string timestamps
    def format_time_parse(time)
      if time.numeric?
        Time.at time
      else
        Time.parse time
      end
    end
    # ---
  end
end
